/* IO module for standard LPT parallel port for PC-Share V3.x
	 V3.0 release - single char buffered io only */

#include <windows.h>
#include <dos.h>
#include "io_ports.h"

#ifdef __cplusplus
    #define __CPPARGS ...
#else
	 #define __CPPARGS
#endif

// port states
#define INPUT 0
#define OUTPUT 1

// PIC addresses
#define PIC0 0x20
#define OCW_1 PIC0+1
#define OCW_2 PIC0

// PIC data
#define EOI 0x20

// lpt offsets
#define DATA_REG 0
#define STATUS_REG 1
#define CTRL_REG 2

// lpt data
#define STROBE_BIT 0x01
#define INT_BIT 16
#define DIR_BIT 32

#define BUF_SIZE 2048

// irqs to cpu ints
const int irq_to_int[] = { 0x8, 0x9, 0xA, 0xB, 0xC, 0xD, 0xE, 0xF } ;

// current direction
int current_dir, base_addr, port_irq ;
BOOL rx_full, tx_full ;

unsigned char rx_buff[BUF_SIZE] ;
int rx_read, rx_write ;

	static unsigned long i = 0 ;

// internals
void init_out_port ( void ) ;
void init_in_port ( void ) ;
inline void send_strobe ( void ) ;

// interrupt handler prototypes
void far interrupt lpt_handler (__CPPARGS) ;
void far interrupt ( *old_isr )(__CPPARGS) ;

/* the DLL stuff */
// Turn off warning: Parameter '' is never used
#pragma argsused
int FAR PASCAL LibMain( HINSTANCE hInstance,
								WORD wDataSegment,
								WORD wHeapSize,
								LPSTR lpszCmdLine )
{

// The startup code for the DLL initializes the local heap(if there is one)
// with a call to LocalInit which locks the data segment.

	 if ( wHeapSize != 0 )
		  UnlockData( 0 );
	 return 1;   // Indicate that the DLL was initialized successfully.
}

// Turn off warning: Parameter '' is never used
#pragma argsused
int FAR PASCAL WEP ( int bSystemExit )
{
	unsigned char irq_mask ;

	disable() ;

	// disable IRQ on PIC
	irq_mask = 1 << port_irq ;
	outportb ( OCW_1, inportb ( OCW_1 ) | irq_mask ) ;

	// disable ints on lpt port
	outportb ( base_addr+CTRL_REG, inportb ( base_addr+CTRL_REG ) | (~INT_BIT) ) ;

	// restore old handler
	setvect ( irq_to_int[port_irq], old_isr ) ;
	enable() ;

	 return 1;
}

// lpt interrupt handler
void far interrupt lpt_handler (__CPPARGS)
{
	unsigned char ctrl_state ;

	// if current dir is input, load byte just rcved into buffer
	if ( current_dir == INPUT )
	{
		rx_buff[rx_write] = inportb ( base_addr+DATA_REG ) ;
		rx_write++ ;
		if ( rx_write == BUF_SIZE )
			rx_write = 0 ;

		// if buffer now full halt any further incoming data by holding
		// off sending data ack - set flag to indicate bfr is now full
		if ( rx_write == rx_read )
			rx_full = TRUE ;
		else
		{
			// get current value in ctrl reg
			ctrl_state = inportb ( base_addr+CTRL_REG ) ;
			// if strobe line is still set ie. we have interrupted mid-way
			// thru a strobe pulse then invert the strobe procedure - first
			// clear it to complete the last strobe and then set it. When the ISR
			// completes, it will then clear it to complete
			if ( ctrl_state & STROBE_BIT )
			{
				outportb ( base_addr+CTRL_REG, ctrl_state & (~STROBE_BIT) ) ;
				outportb ( base_addr+CTRL_REG, ctrl_state | STROBE_BIT ) ;
			}
			else
			{
				// in-lined send_strobe - for speed
				outportb ( base_addr+CTRL_REG, ctrl_state | STROBE_BIT ) ;
				outportb ( base_addr+CTRL_REG, ctrl_state & (~STROBE_BIT) ) ;
			}
		}
	}
	else
	{
		// current dir is output
		// just clear flag - indicate tx buff no longer full
		tx_full = FALSE ;
	}

	// clear irq from pic
	outportb ( OCW_2, EOI ) ;
}

// toggle strobe line of printer port
inline void send_strobe ( void )
{
//	unsigned char ctrl_state ;

//	ctrl_state = inportb ( base_addr+CTRL_REG ) ;

	if ( current_dir == INPUT )
	{
		outportb ( base_addr+CTRL_REG, INT_BIT + DIR_BIT + STROBE_BIT ) ;
		outportb ( base_addr+CTRL_REG, INT_BIT + DIR_BIT ) ;
	}
	else
	{
		outportb ( base_addr+CTRL_REG, INT_BIT + STROBE_BIT ) ;
		outportb ( base_addr+CTRL_REG, INT_BIT  ) ;
	}
//	outportb ( base_addr+CTRL_REG, ctrl_state | STROBE_BIT ) ;
//	outportb ( base_addr+CTRL_REG, ctrl_state & (~STROBE_BIT)) ;
}

// set base addr of port to be used
#pragma argsused
void far _export set_port_base ( int port_base, int irq, int dma )
{
	unsigned char irq_mask ;
	int int_vect ;

	// set port base
	base_addr = port_base ;
	port_irq = irq ;

	// setup buffer pointers
	rx_read = 0 ;
	rx_write = 0 ;
	rx_full = FALSE ;
	tx_full = FALSE ;

	// disable ints
	disable() ;

	// mask for pic
	irq_mask = 1 << irq ;

	// enable lpt interrupts on pic
	outportb ( OCW_1, inportb ( OCW_1 ) & (~irq_mask) ) ;

	// enable ack interrupts on parallel port
	outportb ( base_addr+CTRL_REG, inportb ( base_addr+CTRL_REG ) | INT_BIT ) ;

	// clear strobe bit
	outportb ( base_addr+CTRL_REG, inportb ( base_addr+CTRL_REG ) & (~STROBE_BIT)) ;

	// setup int handler
	int_vect = irq_to_int[irq] ;
	old_isr = getvect ( int_vect ) ;

	// set to point to new handler
	setvect ( int_vect, lpt_handler ) ;

	// clear anything pending
	outportb ( OCW_2, EOI ) ;

	// restore ints
	enable() ;

	// initialise port for input
	init_in_port() ;
}

// get in_port status
int far _export in_port_status ( void )
{
	if ( current_dir != INPUT )
		init_in_port() ;

	// if bfr pointers do not match, or rx bfr is marked as full
	// then data is available
	if ( rx_read != rx_write || rx_full )
		return TRUE ;
	else
		return FALSE ;
}

// read a data byte from port
unsigned char far _export read_data ( void )
{
	unsigned char data_read ;

	if ( current_dir != INPUT )
		init_in_port() ;

	// data available?
	while ( ! in_port_status() )
	{
	}

	// fetch from buffer & update ptrs
	data_read = rx_buff[rx_read] ;
	rx_read++ ;
	if ( rx_read == BUF_SIZE )
		rx_read = 0 ;

	// if rx bfr has been marked as full, clear the flag
	// and send the required strobe pulse to resume data traffic
	if ( rx_full )
	{
		rx_full = FALSE ;
		send_strobe() ;
	}

	// return with data
	return data_read ;
}

// initialise 8255 for input
void init_in_port ( void )
{
	// before doing anything, make sure any data going out has been sent
	while ( out_port_status() ) ;

	current_dir = INPUT ;

	// set port for input
	outportb ( base_addr+CTRL_REG, inportb ( base_addr+CTRL_REG ) | DIR_BIT ) ;

	// compatability with 8255 - do 'reset' pulse
	send_strobe() ;
}

// initialise 8255 for output
void init_out_port ( void )
{
	// set port for output
	outportb ( base_addr+CTRL_REG, inportb ( base_addr+CTRL_REG ) & (~DIR_BIT) ) ;

	current_dir = OUTPUT ;

	// compatability with 8255 - do 'reset' pulse
	send_strobe() ;
}

// write data byte to outport port
void far _export write_data ( unsigned char data_byte )
{

	if ( current_dir != OUTPUT )
		init_out_port() ;

	while ( tx_full )
	{
		i++ ;
	}

	outportb ( base_addr+DATA_REG, data_byte ) ;
	tx_full = TRUE ;
	send_strobe() ;
}

// get status of output port
// returns TRUE if data still due to go out
int far _export out_port_status ( void )
{
	if ( current_dir != OUTPUT )
		init_out_port() ;

	return tx_full ;
}

