/*
HNC INDIVIDUAL PROJECT PC REMOTE SHARER SYSTEM
PACKAGE VT100_EMUL
CREATED ON February 1, 1995, 07:14 PM BY J.BIRD
MOD ON September 09 1995, FOR V2.03 BY J.Bird
MOD ON February 21 1996, FOR V2.04 BY J.Bird
MOD ON March 23 1996, FOR V2.1 BY J.Bird
MOD ON October 29 1996, FOR V3.0 BY J.Bird

Package VT100_Emul is

DESCRIPTION

This package provides the DEC VT100 emulation facility for the project. The
object will recognize all VT100 & VT52 escape sequences, as identified in the
Data Interface Description Document (Ref: 106-4-12, Issue 1, February 1995)
and process them in accordance with this document.

REQUIRED INTERFACE
 None
*/
#include "globals.h"
#include "commemu.h"
#include "vt100_em.h"
/*
Types/Data

-- VT100 sequence level
-- these are the possible processing levels.
type VT100_LEVEL_TYPE is ( NORMAL,
									ESC_DETECT,
									SQR_OR_QUESTION_MARK,
									SQUARE_BRACKET,
									QUESTION_MARK,
									ROUND_BRACKET,
									HASH,
									MISC ) ;
*/
#define NORMAL 0
#define ESC_DETECT 1
#define SQR_OR_QUESTION_MARK 2
#define SQUARE_BRACKET 3
#define QUESTION_MARK 4
#define ROUND_BRACKET 5
#define HASH 6
#define MISC 7
#define ATTRS 8

/*
-- current level
VT100_LEVEL : VT100_LEVEL_TYPE ;
-- numeric parameters
NUMERIC_PARAM : array ( 1 .. 2 ) of integer ;
PARAM_COUNT : integer ;
-- current multiplier for numeric params
MULTIPLIER : integer := 1 ;
-- flag to indicate open/closed round bracket
OPEN_BRACKET_FLAG : boolean ;
// default tab positions
-- valid recognition characters
-- these are the codes that VT100 will detect
-- as the start of an escape sequence by VT_Detect
Control_Code_Table : array :=
											(  Bell,
												Backspace,
												Home,
												Clear,
												Tab,
												Escape ) ;
*/
#define Escape 27
#define Shift_In 0x0F

/*
PROVIDES OPERATIONS
 PROCESS_VT_SEQUENCE
 VT_DETECT
 VT_IN_PROGRESS
*/

/*
INTERNAL OPERATIONS
 PROCESS_SQR_BRACKET_SEQUENCE
 PROCESS_QUESTION_MARK_SEQUENCE
 PROCESS_ROUND_BRACKET_SEQUENCE
 PROCESS_HASH_SEQUENCE
 PROCESS_MISC_SEQUENCE
 PROCESS_NUMERIC_PARAM

END.
*/

// constructor
vt100_emulator::vt100_emulator()
{
	int i ;

	VT100_level = NORMAL ;
	param_count = 0 ;
	multiplier = 1 ;
	open_bracket_flag = FALSE ;
	pending = FALSE ;

	// set default tabs to every 9th column
	for (i=0; i<80 ; i++)
	{
		if ( i%9 == 0 )
			tab_array[i] = 1 ;
		else
			tab_array[i] = 0 ;
	}
}

void vt100_emulator::init_numeric_param ( void )
{
	multiplier = 1 ;
	numeric_param[0] = 0 ;
	numeric_param[1] = 0 ;
	param_count = 0 ;
	current_param = 0 ;
}

/*
HNC INDIVIDUAL PROJECT PC REMOTE SHARER SYSTEM
INTERNAL OPERATION PROCESS_NUMERIC_PARAM
CREATED ON February 1, 1995, 07:14 PM BY J.BIRD
MOD ON October 29 1996, FOR V3.0 BY J.Bird

Procedure Process_Numeric_Param ( Character ) is
*/
void vt100_emulator::process_numeric_param ( char textcr )
/*
DESCRIPTION

This operation is responsible for building up an integer value, stored in the
current Numeric Parameter array from each ASCII number character passed to it.

Each the operation is called, the existing number is multiplied by the next
factor of ten, and the new number added to it.

USES OPERATIONS
 None

Types/Data
 Number : integer ;

PSUEDO CODE

 Numeric_Param ( Param_Count ) := Numeric_Param ( Param_Count ) * Multiplier ;

 Convert_ASCII_Characterr_to_number ;

 Numeric_Param ( Param_Count ) := Numeric_Param ( Param_Count ) + Number ;

 Multiplier := Multiplier * 10 ;

CODE
*/
{
	numeric_param[param_count] = numeric_param[param_count] * multiplier ;
	textcr = textcr - '0' ;
	numeric_param[param_count] = numeric_param[param_count] + textcr ;
	multiplier = multiplier * 10 ;
}
/*
END.
*/

/*
HNC INDIVIDUAL PROJECT PC REMOTE SHARER SYSTEM
INTERNAL OPERATION PROCESS_SQR_BRACKET_SEQUENCE
CREATED ON February 1, 1995, 07:14 PM BY J.BIRD
MOD ON February 21 1996, FOR V2.04 BY J.Bird
MOD ON March 23 1996, FOR V2.1 BY J.Bird
MOD ON November 2 1996, FOR V3.0 BY J.Bird

Procedure Process_Sqr_Bracket_Sequence ( Character ) is
*/
int vt100_emulator::process_sqr_bracket_sequence ( char textcr )
/*
DESCRIPTION

This operation is responsible for handling VT100 sequences starting with an
escape character, and square bracket. The sequence can then take a number of
numeric parameters and an execution code.

The square bracket sequences comprise the majority of VT100 escape sequences.

Sequences starting escape, square bracket, question mark are handled
elsewhere.

USES OPERATIONS
 PROCESS_NUMERIC_PARAM

Types/Data
 None
*/
{
int tab_count ;
/*
PSUEDO CODE

 case character is

	  when '0 .. 9'

			Process_Numeric_Param ( Character ) ;

	  when ';'

			Param_Count := Param_Count + 1 ;
			Multiplier := 1 ;

		 when 'h'

			if Numeric_Param ( 1 ) = 20
			then

				Set_New_Line_Mode ;

			end if ;

			VT100_Level := Normal ;

		 when 'l'

			if Numeric_Param ( 1 ) = 20
			then

				 Set_Line_Feed_Mode ;

			end if ;

			VT100_Level := Normal ;

		 when 'm'

			case Numeric_Param ( 1 ) is

				 when 0

					Turn_Off_All_Attributes ;

				 when 1

					Turn_Bold_On ;

				 when 2

					Turn_Low_Intensity_On ;

				 when 4

					Turn_Underline_Mode_On ;

				 when 5

					Turn_Blinking_Mode_On ;

				 when 7

					 Turn_Reverse_Video_On ;

				 when 8

					 Turn_Invisible_Text_Mode_On ;

			end case ;

			VT100_Level := Normal ;

	  when 'r'

			Top_Line_Of_Window := Numeric_Param ( 1 ) ;
			Bottom_Line_Of_Window := Numeric_Param ( 2 ) ;
			VT100_Level := Normal ;

	  when 'A'

			Cursor_Up Numeric_Param ( 1 ) ;
			VT100_Level := Normal ;

	  when 'B'

			Cursor_Down Numeric_Param ( 1 ) ;
			VT100_Level := Normal ;

	  when 'C'

			Cursor_Right_Numeric_Param ( 1 ) ;
			VT100_Level := Normal ;

		 when 'D'

			Cursor_Left_Numeric_Param ( 1 ) ;
			VT100_Level := Normal ;

	  when 'H' or 'f'

			Set_Cursor Numeric_Param ( 1 ), Numeric_Param ( 2 ) ;
			VT100_Level := Normal ;

	  when 'g'

			if Numeric_Param ( 1 ) = 0
			then

				 Clear_Tab_at_Current_Column ;

			else if Numeric_Param ( 1 ) = 3
			then

				 Clear_all_tabs ;

			end if ;

			VT100_Level := Normal ;

		 when 'K'

			case Numeric_Param ( 1 ) is

				 when 0

					Clear_Line_from_Cursor_Right ;

				 when 1

					Clear_Line_from_Cursor_Left ;

				 when 2

					Clear_Entire_Line ;

				 end case ;

			VT100_Level := Normal ;

		 when 'J'

			case Numeric_Param ( 1 ) is

				 when 0

					  Clear_Screen_from_Cursor_Down ;

				 when 1

					  Clear_Screen_from_Cursor_Up ;

				 when 2

					  Clear_Entire_Screen ;

			end case ;

			VT100_Level := Normal ;

		 when 'y'

			if Numeric_Param ( 1 ) = 2
			then

				 case Numeric_Param ( 2 ) is

					  when 1

							Execute_Power_Up_Test ;

					  when 2

							Execute_Loopback_Test ;

					  when 9

							Repeat_Power_Up_Test ;

					 when 10

							 Repeat_Loopback_Test ;

				 end case ;

			end if ;

			VT100_Level := Normal ;

		when 'q'

			case Numeric_Param ( 1 ) is

				 when 0

					  Turn_Off_LEDS ;

				 when 1 .. 4

					  Turn_On_Required_LED ;

			end case ;

			VT100_Level := Normal ;

	  when others

			VT100_Level := Normal ;

 end case ;

CODE
*/
	 if ( textcr >= '0' && textcr <= '9' )
	 {
		  process_numeric_param ( textcr ) ;
	 }
	 else
	 {
		switch ( textcr )
		{
			case ';' :
			{
				param_count++ ;
				multiplier = 1 ;
				break ;
			}

			case 'h' :
			{
					VT100_level = NORMAL ;
					break ;
			}

			case 'l' :
			{
					VT100_level = NORMAL ;
					break ;
			}
			case 'm' :
			{
				// init param counter on first time thru
				if ( !pending )
					current_param=0 ;

				// if more than 1 param, set flags to indicate
				// further processing needed to complete
				if ( param_count > 0 )
				{
					pending = TRUE ;
					pending_char = textcr ;
					param_count-- ;
				}
				else
				{
					pending = FALSE ;
					VT100_level = NORMAL ;
				}

				current_param++ ;
				// process params
				switch ( numeric_param[current_param-1] )
				{
					case 0 :
					{
						return NO_ATTRS ;
					}
					case 1:
					{
						return BOLD_ON ;
					}
					case 2:
					{
						return BOLD_OFF ;
					}
					case 4:
					{
						return UNDERLINE_ON ;
					}
					case 5:
					{
						return BLINK_ON ;
					}
					case 7:
					{
						return REVERSE_ON ;
					}
					case 8:
					{
						return INVISIBLE_ON ;
					}
				}
				return NO_ACTION ;
			}

			case 'r' :
			{
				VT100_level = NORMAL ;
				return NO_ACTION ;
			}

			case 'A' :
			{
				VT100_level = NORMAL ;
				new_y = numeric_param[0] ;
				return CURSOR_UP ;
			}

			case 'B' :
			{
				VT100_level = NORMAL ;
				new_y = numeric_param[0] ;
				return CURSOR_DOWN ;
			}

			case 'C' :
			{
				VT100_level = NORMAL ;
				new_x = numeric_param[0] ;
				return CURSOR_RIGHT ;
			}

			case 'D' :
			{
				VT100_level = NORMAL ;
				new_x = numeric_param[0] ;
				return CURSOR_LEFT ;
			}

			case 'H' :
			{
				VT100_level = NORMAL ;
				new_x = numeric_param[1]-1 ;
				new_y = numeric_param[0]-1 ;
				return SET_CURSOR ;
			}

			case 'f' :
			{
				VT100_level = NORMAL ;
				new_x = numeric_param[1]-1 ;
				new_y = numeric_param[0]-1 ;
				return SET_CURSOR ;
			}

			case 'g' :
			{
				VT100_level = NORMAL ;

				if ( numeric_param[0] == 0 )
					return CLEAR_TAB ;
				else
				{
					if ( numeric_param[0] == 3 )
					{
						for ( tab_count = 0 ; tab_count < 79 ; tab_count++ )
						{
							tab_array[tab_count] = 0 ;
						}
					}
				}
				return NO_ACTION ;
			}

			case 'K' :
			{
				switch ( numeric_param[0] )
				{
					case 0 :
					{
						VT100_level = NORMAL ;
						return CLEAR_EOL ;
					}

					case 1 :
					{
						VT100_level = NORMAL ;
						return CLEAR_LEFT ;
					}

					case 2 :
					{
						VT100_level = NORMAL ;
						return CLEAR_LINE ;
					}
				}
				VT100_level = NORMAL ;
				return NO_ACTION ;

			}

			case 'J' :
			{
				switch ( numeric_param[0] )
				{
					case 0 :
					{
						VT100_level = NORMAL ;
						return CLEAR_EOS ;
					}
					case 1 :
					{
						VT100_level = NORMAL ;
						return CLEAR_TOS ;
					}
					case 2 :
					{
						VT100_level = NORMAL ;
						return CLEAR_AOS ;
					}
				}
				VT100_level = NORMAL ;
				return NO_ACTION ;
			}

			case 'y' :
			{
				switch ( numeric_param[0] )
				{
					case 1 :
					{
						break ;
					}

					case 2 :
					{
						break ;
					}

					case 9 :
					{
						break ;
					}

					case 10 :
					{
						break ;
					}

				}
				VT100_level = NORMAL ;
				return NO_ACTION ;

			}

			case 'q' :
			{
				switch ( numeric_param[0] )
				{
					case 0 :
					{
						break ;
					}
					case 1 :
					{
						break ;
					}
					case 2 :
					{
						break ;
					}
					case 3 :
					{
						break ;
					}
					case 4 :
					{
						break ;
					}
				}
				VT100_level = NORMAL ;
				return NO_ACTION ;
			}

			default :
			{
				VT100_level = NORMAL ;
				return NO_ACTION ;
			}
		}
	 }
	 return NO_ACTION ;
}
/*
END.
*/

/*
HNC INDIVIDUAL PROJECT PC REMOTE SHARER SYSTEM
INTERNAL OPERATION PROCESS_QUESTION_MARK_SEQUENCE
CREATED ON February 1, 1995, 07:14 PM BY J.BIRD
MOD ON February 21 1996, FOR V2.1 BY J.Bird
MOD ON November 17 1996, FOR V3.0 BY J.Bird

Procedure Process_Question_Mark_Sequence ( Character ) is
*/
int vt100_emulator::process_question_mark_sequence ( char textcr )
/*
DESCRIPTION

This operation processes a sub-set of the square bracket sequences, those
starting escape, square bracket, question mark. The following codes then give
a numeric id followed by a code to determine a given setting (true/false).

These sequences define terminal settings.

USES OPERATIONS
 PROCESS_NUMERIC_PARAM

Types/Data

PSUEDO CODE

 case character is

	  when '0..9'

				 Process_Numeric_Param ( Character ) ;

	  when 'h'

	  case Numeric_Param ( 1 ) is

			when 1

				 Set_Cursor_Key_to_Application ;

			when 3

				 Set_Columns_to_132 ;

			when 4

				 Set_Smooth_Scrolling ;

			when 5

				 Set_Reverse_Video_On_Screen ;

			when 6

				 Set_Origin_to_Relative ;

			when 7

				 Set_Auto_Wrap_Mode ;

			when 8

				 Set_Auto_Repeat_Mode ;

			when 9

				 Set_Interlacing_Mode ;

	  end case ;
	  VT100_Level := Normal ;

	  when 'l'

			case Numeric_Param ( 1 ) is

				 when 1

					  Set_Cursor_Key_to_Cursor ;

				 when 2

					  Set_VT52 ;

				 when 3

					  Set_Number_of_Columns_to_80 ;

				 when 4

					  Set_Jump_Scrolling ;

				 when 5

					  Set_Normal_Video_On_Screen ;

				 when 6

					  Set_Origin_to_Absolute ;

				 when 7

					  Reset_Auto_Wrap_Mode ;

				 when 8

					  Reset_Auto_Repeat_Mode ;

				 when 9

					  Reset_Interlacing_Mode ;

			end case ;

			VT100_Level := Normal ;

	  when others

			VT100_Level := Normal ;

 end case ;

CODE
*/

{
	 if ( textcr >= '0' && textcr <= '9' )
	 {
		process_numeric_param ( textcr ) ;
	 }
	 else
	 {
		  switch ( textcr )
		  {
				case 'h' :
				{
					 switch ( numeric_param[0] )
					 {
						case 1 :
						{
							break ;
						}

						case 2 :
						{
							break ;
						}

						case 3 :
						{
							VT100_level = NORMAL ;
							return WIDTH_132 ;
						}

						case 4 :
						{
							break ;
						}

						case 5 :
						{
							VT100_level = NORMAL ;
							return REVERSE_ON ;
						}

						case 6 :
						{
							break ;
						}

						case 7 :
						{
							VT100_level = NORMAL ;
							return AUTO_WRAP ;
						}

						case 8 :
						{
							break ;
						}

						case 9 :
						{
							break ;
						}
					 }
					 VT100_level = NORMAL ;
					 return NO_ACTION ;
				}

				case 'l' :
				{
					 switch ( numeric_param[0] )
					 {
						  case 1 :
						  {
								break ;
						  }

						  case 2 :
						  {
								break ;
						  }

						  case 3 :
						  {
								VT100_level = NORMAL ;
								return WIDTH_80 ;
						  }

						  case 4 :
						  {
								break ;
						  }

						  case 5 :
						  {
								VT100_level = NORMAL ;
								return REVERSE_OFF ;
						  }

						  case 6 :
						  {
								break ;
						  }

						  case 7 :
						  {
								VT100_level = NORMAL ;
								return RESET_WRAP ;
						  }

						  case 8 :
						  {
								break ;
						  }

							case 9 :
						  {
								break ;
						  }
					 }
					 VT100_level = NORMAL ;
				}

				default :
				{
					 VT100_level = NORMAL ;
				}
		  }
	 }
	 return NO_ACTION ;
}

/*
END.
*/

/*
HNC INDIVIDUAL PROJECT PC REMOTE SHARER SYSTEM
INTERNAL OPERATION PROCESS_ROUND_BRACKET_SEQ
CREATED ON February 1, 1995, 07:14 PM BY J.BIRD
MOD ON November 17 1996, FOR V3.0 BY J.Bird

Procedure Process_Round_Bracket_Seq ( Character ) is
*/
int vt100_emulator::process_round_bracket_seq ( char textcr )
/*
DESCRIPTION

This operation is responsible for processing VT100 escape sequences comprising
an escape character followed by an open or closed round bracket and a single
execution code. The execution code is the character passed to the procedure.

The round bracket sequences are primarily used to select character & font
information, and depending on whether an open or closed bracket is used will
determine which font/character set is used.

Upon execution of the operation, the VT100_Level is returned to Normal.

USES OPERATIONS
 None

Types/Data
 None

PSUEDO CODE

 case character is

	  when 'A'

			Set_UK_G_Character_Set ;

	  when 'B'

			Set_US_G_Character_Set ;

	  when '0'

			Set_G_Special_Chars_and_Line_Set ;

	  when '1'

			Set_G_Alternate_Character_ROM ;

	  when '2'

			Set_G_Alt_Char_ROM_and_Spec_Graphics ;

 end case ;

 VT100_Level := Normal ;

CODE
*/
{
	 switch ( textcr )
	 {
		  case 'A' :
		  {
				VT100_level = NORMAL ;
				return NO_ACTION ;
		  }

		  case 'B' :
		  {
				VT100_level = NORMAL ;
				return NO_ACTION ;
		  }

		  case '0' :
		  {
				VT100_level = NORMAL ;
				return NO_ACTION ;
		  }

		  case '1' :
		  {
				VT100_level = NORMAL ;
				return NO_ACTION ;
		  }

		  case '2' :
		  {
				VT100_level = NORMAL ;
				return NO_ACTION ;
		  }

		  default :
		  {
				VT100_level = NORMAL ;
				return NO_ACTION ;
		  }

	 }
}

/*
END.
*/

/*
HNC INDIVIDUAL PROJECT PC REMOTE SHARER SYSTEM
INTERNAL OPERATION PROCESS_HASH_SEQUENCE
CREATED ON February 1, 1995, 07:14 PM BY J.BIRD
MOD ON November 17 1996, FOR V3.0 BY J.Bird

Procedure Process_Hash_Sequence ( Character ) is
*/
int vt100_emulator::process_hash_sequence ( char textcr )
/*
DESCRIPTION

This operation is responsible for processing VT100 escape sequences starting
with an escape character followed by a hash symbol and an execution code. The
execution code is the character passed into this procedure.

The hash set of escape codes are responsible for character height/width.

Upon completion of the execution code, the VT100_Level is set to Normal.

USES OPERATIONS
 None

Types/Data
 None

PSUEDO CODE

 case character is

	  when '3'

			Set_Top_Half_Double_Height_Letters ;

	  when '4'

			Set_Bottom_Half_Double_Height_Letters ;

	  when '5'

			Set_Single_Width_Single_Height_Letters ;

	  when '6'

			Set_Double_Width_Single_Height_Letters ;

	  when '8'

			Screen_Alignment_Display ;

 end case ;

 VT100_Level := Normal ;

CODE
*/
{
	 switch ( textcr )
	 {
		  case '3' :
		  {
				VT100_level = NORMAL ;
				return NO_ACTION ;
		  }

		  case '4' :
		  {
				VT100_level = NORMAL ;
				return NO_ACTION ;
		  }

		  case '5' :
		  {
				VT100_level = NORMAL ;
				return NO_ACTION ;
		  }

		  case '6' :
		  {
				VT100_level = NORMAL ;
				return NO_ACTION ;
		  }

		  case '8' :
		  {
				VT100_level = NORMAL ;
				return NO_ACTION ;
		  }

		  default :
		  {
				VT100_level = NORMAL ;
				return NO_ACTION ;
		  }
	 }
}
/*
END.
*/

/*
HNC INDIVIDUAL PROJECT PC REMOTE SHARER SYSTEM
INTERNAL OPERATION PROCESS_MISC_SEQUENCE
CREATED ON February 1, 1995, 07:14 PM BY J.BIRD
MOD ON February 21 1996, FOR V2.1 BY J.Bird
MOD ON October 29 1996, FOR V3.0 BY J.Bird

Procedure Process_Misc_Sequence ( Character ) is
*/
int vt100_emulator::process_misc_sequence ( char textcr )
/*
DESCRIPTION

This operation handles any VT100 escape sequences which do not fall into one
of the other sequences (square bracket,round bracket or hash). They all
consist of a single character execution code.

Upon completing the code, the VT100_Level is returned to Normal.

USES OPERATIONS
 None

Types/Data
 None
*/
{
/*
PSUEDO CODE

 case character is

	  when '='

			Set_Alternate_Keypad ;

	  when '>'

			Set_Numeric_Keypad ;

	  when 'N'

			Set_Single_Shift_2 ;

	  when 'O'

			Set_Single_Shift_3 ;

	  when 'D'

			Scroll_Up_1_Line ;

	  when 'M'

			Scroll_Down_1_Line ;

	  when 'E'

			Move_to_Next_Line ;

	  when '7'

			Save_Cursor_Position_and_Attributes ;

	  when '8'

			Restore_Cursor_Position_and_Attributes ;

	  when 'H'

			Set_TAB ;

	  when 'c'

			Reset_Terminal ;

 end case ;

 VT100_Level := Normal ;

CODE
*/
	 switch ( textcr )
	 {
		  case '=' :
		  {
				VT100_level = NORMAL ;
				return NO_ACTION ;
		  }

		  case '>' :
		  {
				VT100_level = NORMAL ;
				return NO_ACTION ;
		  }

		  case 'N' :
		  {
				VT100_level = NORMAL ;
				return NO_ACTION ;
		  }

		  case 'O' :
		  {
				VT100_level = NORMAL ;
				return NO_ACTION ;
		  }

		  case 'D' :
		  {
				VT100_level = NORMAL ;
				return NO_ACTION ;
		  }

		  case 'M' :
		  {
				VT100_level = NORMAL ;
				return NO_ACTION ;
		  }

		  case 'E' :
		  {
				VT100_level = NORMAL ;
				return CURSOR_DOWN ;
		  }

		  case '7' :
		  {
				VT100_level = NORMAL ;
				return SAVE_ATTRS ;
		  }

		  case '8' :
		  {
				VT100_level = NORMAL ;
				return REST_ATTRS ;
		  }

		  case 'H' :
		  {
				VT100_level = NORMAL ;
				return SET_TAB ;
		  }

		  case 'c' :
		  {
				VT100_level = NORMAL ;
				return RESET_TERM ;
		  }

		  default :
		  {
				VT100_level = NORMAL ;
				return NO_ACTION ;
		  }
	 }
}
/*
END.
*/

/*
HNC INDIVIDUAL PROJECT PC REMOTE SHARER SYSTEM
PROVIDED OPERATION PROCESS_VT_SEQUENCE
CREATED ON February 1, 1995, 07:14 PM BY J.BIRD
MOD ON February 21 1996, FOR V2.1 BY J.Bird
MOD ON October 29 1996, FOR V3.0 BY J.Bird

Procedure Process_VT_Sequence ( Character ) is
*/
int vt100_emulator::process_emulation_sequence ( char textcr )
/*
DESCRIPTION

This operation handles the top level processing of a VT100 escape sequence.
The incoming character is compared against a set of control codes which must
be processed. Any control code except escape can be processed immediately. If
an escape character is detected, the VT100_Level is set to Esc_Detect. The
next level of processing is based upon the next character which can be:

-- a square bracket, round bracket or hash character starting sequence.
-- a miscellaneous sequence.

If a square bracket is detected, then an intermediate state is entered to
determine if a question mark sequence is occuring.

USES OPERATIONS
 PROCESS_SQR_BRACKET_SEQUENCE
 PROCESS_QUESTION_MARK_SEQUENCE
 PROCESS_ROUND_BRACKET_SEQUENCE
 PROCESS_HASH_SEQUENCE
 PROCESS_MISC_SEQUENCE

Types/Data
 None
{
*/
/*
PSUEDO CODE

 case VT100_Level is

		 when Normal

			case character is

				 when bell

					  Output_bell ;

				 when backspace

					  Output_delete_char ;

				 when home

					  Set_cursor_to_top_left_corner ;

				 when clear

					  Clear_text_screen ;

				 when tab

					  Move_to_next_tab_position ;

				 when escape

					  VT100_Level := Esc_Detect ;

			end case ;

	  when Esc_Detect

				 case character is

			when '['

				 VT100_Level := Sqr_or_question_mark  ;
				 Multiplier := 1 ;
				 Param_Count := 1 ;

			when '('

		VT100_Level := Round_Bracket ;
				 Open_Bracket_Flag := True ;

			when ')'

				 VT100_Level := Round_Bracket ;
				 Open_Bracket_Flag := False ;

			when '#'

				 VT100_Level := Hash ;

			when others

				 VT100_Level := Misc ;

			end case ;

	  when Sqr_or_question_mark

			if character = '?'
			then

				 VT100_Level := Question_Mark ;

			else

				 VT100_Level := Square_Bracket ;
				 Process_Sqr_Bracket_Sequence ( Character ) ;

			end if ;

	  when Square_Bracket

			Process_Sqr_Bracket_Sequence ( Character ) ;

	  when Question_Mark

			Process_Question_Mark_Sequence ( Character ) ;

	  when Round_Bracket

			Process_Round_Bracket_Sequence ( Character ) ;

	  when Hash

			Process_Hash_Sequence ( Character ) ;

	  when Misc

			Process_Misc_Sequence ( Character ) ;

 end case

CODE
*/
{
	 switch ( VT100_level )
	 {
		  case NORMAL :

				switch ( textcr )
				{
					case Escape :
						VT100_level = ESC_DETECT ;
						return NO_ACTION ;

					case Shift_In :
						return NO_ACTION ;

					default :
						return process_comm_emulator_sequence ( textcr ) ;
				}

		  case ESC_DETECT :
				switch ( textcr )
				{
					 case '[' :
					 {
							init_numeric_param() ;
							VT100_level = SQR_OR_QUESTION_MARK ;
							return NO_ACTION ;
					 }

					 case '(' :
					 {
							VT100_level = ROUND_BRACKET ;
							open_bracket_flag = TRUE ;
							return NO_ACTION ;
					 }

					 case ')' :
					 {
							VT100_level = ROUND_BRACKET ;
							open_bracket_flag = FALSE ;
							return NO_ACTION ;
					 }

					 case '#' :
					 {
							VT100_level = HASH ;
							return NO_ACTION ;
					 }

					default:
					{
							VT100_level = MISC ;
							return process_misc_sequence ( textcr ) ;
					}
				}

		  case SQR_OR_QUESTION_MARK :
		  {

				if ( textcr == '?' )
				{
						VT100_level = QUESTION_MARK ;
						return NO_ACTION ;
				}
				else
				{
						VT100_level = SQUARE_BRACKET ;
						return process_sqr_bracket_sequence ( textcr ) ;
				}
		  }

		  case SQUARE_BRACKET :
		  {
				return process_sqr_bracket_sequence ( textcr ) ;
		  }

		  case QUESTION_MARK :
		  {
				return process_question_mark_sequence ( textcr ) ;
		  }

		  case ROUND_BRACKET :
		  {
				return process_round_bracket_seq ( textcr ) ;
		  }

		  case HASH :
		  {
				return process_hash_sequence ( textcr ) ;
		  }

		  case MISC :
		  {
				return process_misc_sequence ( textcr ) ;
		  }
	 }
	 return NO_ACTION ;
}
/*
END.
*/

/*
HNC INDIVIDUAL PROJECT PC REMOTE SHARER SYSTEM
PROVIDED OPERATION VT_DETECT
CREATED ON February 1, 1995, 07:14 PM BY J.BIRD
MOD ON March 23 1996, FOR V2.1 BY J.Bird
MOD ON October 29 1996, FOR V3.0 BY J.Bird

Function VT_Detect ( Character ) return boolean is
*/
int vt100_emulator::emulation_detect ( char textcr )
/*

DESCRIPTION

This operation determines if the character passed could be part of an escape
sequence by referring it to the Control Code lookup table.

USES OPERATIONS
 None

Types/Data
 Control_Code : boolean ;


PSUEDO CODE

 Control_Code := false ;

 Loop_through_Control_Code_Table

	  if Character = Value_in_Control_Code_Table
	  then

			Control_Code := true ;

	  end if ;

 End loop ;

 return Control_Code ;

CODE
*/
{
	 if ( comm_emulator_detect ( textcr ) || textcr == Escape || textcr == Shift_In )
		 return TRUE ;

	 return FALSE ;
}
/*
END.
*/

/*
HNC INDIVIDUAL PROJECT PC REMOTE SHARER SYSTEM
PROVIDED OPERATION VT_IN_PROGRESS
CREATED ON February 1, 1995, 07:14 PM BY J.BIRD
MOD ON October 29 1996, FOR V3.0 BY J.Bird

Function VT_In_Progress return boolean is
*/
int vt100_emulator::emulation_in_progress ( void )
/*
DESCRIPTION

This operation determines if a VT100 sequence is in progress, and that the
next character should be passed to the VT100_Emul for processing. The
VT100_Level variabl defines the current processing level, which is only set to
normal if an escape sequence is not in progress.

USES OPERATIONS
 None

Types/Data
 None

PSUEDO CODE

 if VT100_Level = Normal
 then

	  return false ;

 else

	  return true ;

 end if ;

CODE
*/
{
	 if ( VT100_level == NORMAL )
		return FALSE ;
	 else
		return TRUE ;

}
/*
END.
*/

// this operation returns the nearest tab column based
// on the current_x cursor position
int vt100_emulator::find_next_tab ( int current_x )
{
	int tab_pos ;

	for ( tab_pos=current_x;tab_pos<80;tab_pos++ )
		if ( tab_array[tab_pos] )
			return tab_pos ;
	return 0 ;
}

// this operation sets a tab at the specified column
void vt100_emulator::modify_tab ( int column, char set_flag )
{
	tab_array[column] = set_flag ;
}

// determines if a vt sequence is still pending completion.
// if it is not, returns NO_ACTION else returns the vt operation code
int vt100_emulator::emulation_sequence_pending ( void )
{
	if (!pending)
		return NO_ACTION ;
	else
	{
		switch ( VT100_level )
		{
		  case SQUARE_BRACKET :
				return process_sqr_bracket_sequence ( pending_char ) ;
		  case QUESTION_MARK :
				return process_question_mark_sequence ( pending_char ) ;
		  case ROUND_BRACKET :
				return process_round_bracket_seq ( pending_char ) ;
		  case HASH :
				return process_hash_sequence ( pending_char ) ;
		  case MISC :
				return process_misc_sequence ( pending_char ) ;
		  default :
				return NO_ACTION ;
		}
	}
}
